;; Line Sensor - Copyright 2016 by Michael Kohn
;; Email: mike@mikekohn.net
;;   Web: http://www.mikekohn.net/
;;
;; Read from a sensor (light or IR?) with an ADC or using a
;; timer for the digital IR sensor if that's what works.

.include "tn85def.inc"
.avr8

; CKSEL = 0001
; CLKPS = 0000
; CKDIV8 = 0

; This should be default.. 1 MHz
; FUSE EXTD = 0xff
; FUSE HIGH = 0xdf
; FUSE LOW =  0x62

HIGH_COUNT equ 3
THRESHOLD_ON equ 2

; r0  = 0
; r1  = 1
; r14 =
; r15 = 255
; r16 =
; r17 =
; r18 =
; r19 = interrupt counter
; r20 = dark_count
; r21 =
; r22 =
; r23 = HIGH_COUNT
; r24 =
; r25 =
; r26 =
; r27 =
; r30 =
; r31 =
;

.org 0x000
  rjmp start
.org 0x00a
  rjmp timer_interrupt

start:
  ;; Interrupts off
  cli

  ;; Set up stack ptr
  ;ldi r17, RAMEND>>8
  ;out SPH, r17
  ldi r17, RAMEND&255
  out SPL, r17

  ;; r0 = 0, r1 = 1, r15 = 255
  eor r0, r0
  eor r1, r1
  inc r1
  ldi r17, 0xff
  mov r15, r17

  ldi r19, 0
  ldi r25, 15 ; servo centered 

  ;; Set up PORTB
  ;; PB0 = Output
  ;; PB2 = analog input (ADC1)
  ldi r17, 1
  out DDRB, r17             ; PB0 is output
  ldi r17, 0
  out PORTB, r17            ; Everything off to start

  ;; Setup TIMER0
  ldi r17, 200                   ; 200us seconds per interrupt @ 1MHz
  ;ldi r17, 80                     ;  10us seconds per interrupt @ 8MHz
  out OCR0A, r17

  ldi r17, (1<<OCIE0A)
  out TIMSK, r17                 ; enable interrupt compare A 
  ldi r17, (1<<WGM01)
  out TCCR0A, r17                ; normal counting (0xffff is top, count up)
  ldi r17, (1<<CS00)             ; CTC OCR0A  Clear Timer on Compare
  out TCCR0B, r17                ; prescale = 1 from clock source

  ;; Setup ADC
  ldi r17, (1<<ADLAR)|(1<<MUX0)   ; left adjust
  out ADMUX, r17
  ;ldi r17, (1<<ADEN)|(1<<ADATE)|(1<<ADSC)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0)
  ldi r17, (1<<ADEN)|(1<<ADATE)|(1<<ADSC)
  out ADCSRA, r17
  ldi r17, (1<<ADC1D)
  out DIDR0, r17

  ;; Bump CPU speed to 8MHz by getting rid of /4 prescaler
  ;ldi r17, 0x80
  ;out CLKPR, r17
  ;out CLKPR, r0

  ;; This is just for debugging the interrupt speed
  mov r16, r0

  ;; dark_count = 0
  mov r20, r0
  ;; r23 is HIGH_COUNT
  ldi r23, HIGH_COUNT

  ; Interrupts on
  sei

main:
  ;rjmp read_sensor
  rjmp main

;; For digital IR sensor
read_sensor:
  ; Charge sensor
  sbi DDRB, 1
  sbi PORTB, 1
  ldi r21, 0
read_sensor_wait_for_charge:
  cpi r21, 2
  brlo read_sensor_wait_for_charge

  ; Read sensor to see if it's charged
  cbi DDRB, 1
  cbi PORTB, 1
  ldi r21, 0
read_sensor_wait_for_discharge:
  cpi r21, 30
  brlo read_sensor_wait_for_discharge

  in r22, PINB

  sbrs r22, 1
  cbi PORTB, 0
  sbrc r22, 1
  sbi PORTB, 0

  rjmp read_sensor

timer_interrupt_digital:
  in r7, SREG
  inc r21
  out SREG, r7
  reti

timer_interrupt_test:
  inc r16
  cpi r16, 5
  breq debug_led_on
  cpi r16, 10
  breq debug_led_off
  rjmp exit_interrupt 
debug_led_on:
  sbi PORTB, PB0
  rjmp exit_interrupt 
debug_led_off:
  cbi PORTB, PB0
  mov r16, r0
  rjmp exit_interrupt 

timer_interrupt:
  ;in r7, SREG
  ;; Read current ADC value
  in r18, ADCH
  ;ldi r18, 51
  cpi r18, 150
  brlo count_up
count_down:
  cpse r20, r0
  dec r20
  rjmp check_threshold
count_up:
  ;; if dark_count != HIGH_COUNT then dark_count++
  cpse r20, r23
  inc r20
check_threshold:
  cpi r20, THRESHOLD_ON
  brlo turn_off_led
turn_on_led:
  sbi PORTB, PB0
  rjmp exit_interrupt
turn_off_led:
  cbi PORTB, PB0
exit_interrupt:
  ;; Trigger new value
  out ADCSRB, r0
  ;out SREG, r7
  reti

signature:
.db "Line Sensor - Copyright 2016 - Michael Kohn - Version 0.01",0



